# coding: utf-8

"""
    Verity471 Unified API

      - creds: 1.0.1
  - indicators: 1.0.0
  - malware: 1.0.0
  - reports: 1.0.0
  - sources: 1.0.2
  - actors: 1.0.1
  - watchers: 1.0.0
  - observables: 1.0.0
  - entities: 1.0.0
  - girs: 1.0.0


    The version of the OpenAPI document: 1.1.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class ReportType(str, Enum):
    """
    Type of the related report
    """

    """
    allowed enum values
    """
    FINTEL = 'fintel'
    INFO_REPORT = 'info_report'
    BREACH_ALERT = 'breach_alert'
    SPOT_REPORT = 'spot_report'
    VULNERABILITY_REPORT = 'vulnerability_report'
    MALWARE_REPORT = 'malware_report'
    GEOPOL_REPORT = 'geopol_report'
    SITUATION_REPORT = 'situation_report'
    NEWS_REPORT = 'news_report'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ReportType from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    Verity471 Unified API

      - creds: 1.0.1
  - indicators: 1.0.0
  - malware: 1.0.0
  - reports: 1.0.0
  - sources: 1.0.2
  - actors: 1.0.1
  - watchers: 1.0.0
  - observables: 1.0.0
  - entities: 1.0.0
  - girs: 1.0.0


    The version of the OpenAPI document: 1.1.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class AdmiraltyCode(str, Enum):
    """
    Admiralty code indicating source reliability and confidence
    """

    """
    allowed enum values
    """
    A1 = 'A1'
    A2 = 'A2'
    A3 = 'A3'
    A4 = 'A4'
    A5 = 'A5'
    A6 = 'A6'
    B1 = 'B1'
    B2 = 'B2'
    B3 = 'B3'
    B4 = 'B4'
    B5 = 'B5'
    B6 = 'B6'
    C1 = 'C1'
    C2 = 'C2'
    C3 = 'C3'
    C4 = 'C4'
    C5 = 'C5'
    C6 = 'C6'
    D1 = 'D1'
    D2 = 'D2'
    D3 = 'D3'
    D4 = 'D4'
    D5 = 'D5'
    D6 = 'D6'
    E1 = 'E1'
    E2 = 'E2'
    E3 = 'E3'
    E4 = 'E4'
    E5 = 'E5'
    E6 = 'E6'
    F1 = 'F1'
    F2 = 'F2'
    F3 = 'F3'
    F4 = 'F4'
    F5 = 'F5'
    F6 = 'F6'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AdmiraltyCode from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    Verity471 Unified API

      - creds: 1.0.1
  - indicators: 1.0.0
  - malware: 1.0.0
  - reports: 1.0.0
  - sources: 1.0.2
  - actors: 1.0.1
  - watchers: 1.0.0
  - observables: 1.0.0
  - entities: 1.0.0
  - girs: 1.0.0


    The version of the OpenAPI document: 1.1.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class ConfidenceLevel(str, Enum):
    """
    Confidence level, e.g. high, medium, low
    """

    """
    allowed enum values
    """
    HIGH = 'high'
    MEDIUM = 'medium'
    LOW = 'low'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ConfidenceLevel from a JSON string"""
        return cls(json.loads(json_str))





import datetime
from typing import Union, NamedTuple, Optional, List

from stix2 import Identity
import pycti

from verity471.verity_stix.constants import INTEL_471


class STIXMapperSettings(NamedTuple):
    client: Union['verity471', None] = None
    api_client: Union['ApiClient', None] = None
    # Get full report's content, including inline images, using additional API call.
    # Only if given report from search endpoint is truncated or has inline images.
    report_full_content: bool = True
    ioc_opencti_score: Optional[int] = None


class StixObjects:
    """
    Helper class for collecting unique STIX instances (by STIX ID)
    """
    def __init__(self, objects: Union[List, None]=None):
        self._container = {}
        if objects:
            self.extend(objects)

    def add(self, item):
        item_id = item.id
        if item_id not in self._container:
            self._container[item_id] = item

    def extend(self, __iterable):
        for i in __iterable:
            self.add(i)

    def get(self):
        return self._container.values()

    def __bool__(self):
        return bool(self._container)


author_name = f"{INTEL_471} Inc."
author_identity = Identity(
    id=pycti.Identity.generate_id(author_name, identity_class="organization"),
    name=author_name,
    identity_class="organization",
    created=datetime.datetime(2022, 1, 1),
    modified=datetime.datetime(2022, 1, 1),
)



# coding: utf-8

"""
    Verity471 Unified API

      - creds: 1.0.1
  - indicators: 1.0.0
  - malware: 1.0.0
  - reports: 1.0.0
  - sources: 1.0.2
  - actors: 1.0.1
  - watchers: 1.0.0
  - observables: 1.0.0
  - entities: 1.0.0
  - girs: 1.0.0


    The version of the OpenAPI document: 1.1.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from verity471.models.classification import Classification
from verity471.models.country_profile_response import CountryProfileResponse
from verity471.models.entities import Entities
from verity471.models.geopol_report_sub_type import GeopolReportSubType
from verity471.models.intelligence_estimate_response import IntelligenceEstimateResponse
from verity471.models.links import Links
from verity471.models.report_attachment import ReportAttachment
from verity471.models.report_content import ReportContent
from verity471.models.report_location import ReportLocation
from verity471.models.reports_victim_response import ReportsVictimResponse
from verity471.models.significant_activity import SignificantActivity
from verity471.models.sources_response import SourcesResponse
from verity471.models.tension_point_response import TensionPointResponse
from typing import Optional, Set, TYPE_CHECKING
from typing_extensions import Self

if TYPE_CHECKING:
    # Only for type checkers; does not run at runtime.
    from verity471.verity_stix import STIXMapperSettings


class GeopolReportDetailsResponse(BaseModel):
    """
    Minimal schema for Geopolitical report response with keys, types, and descriptions
    """ # noqa: E501
    active_event: Optional[StrictBool] = Field(default=None, description="Report related active event")
    attachments: Optional[List[ReportAttachment]] = Field(default=None, description="List of attachments related to the report")
    body: Optional[StrictStr] = Field(default=None, description="Main content body of the report")
    classification: Classification
    country_profiles: Optional[List[CountryProfileResponse]] = Field(default=None, description="Report related country profiles")
    creation_ts: StrictStr = Field(description="Timestamp when the report was published first time")
    derived_entities: Optional[List[Entities]] = Field(default=None, description="List of entities derived from the report")
    entities: Optional[List[Entities]] = Field(default=None, description="List of entities mentioned in the report")
    id: StrictStr = Field(description="Unique identifier of the report")
    information_ts: StrictStr = Field(description="Timestamp of information date")
    intelligence_estimate: Optional[IntelligenceEstimateResponse] = None
    is_sensitive_source: Optional[StrictBool] = Field(default=None, description="Indicates if the report contains sensitive source derived information")
    is_truncated: Optional[StrictBool] = Field(default=None, description="True when the body field was omitted due to exceeding size limit; fetch full report by ID if needed")
    last_updated_ts: StrictStr = Field(description="Timestamp of last report update")
    links: Links
    locations: Optional[List[ReportLocation]] = Field(default=None, description="List of locations related to the report")
    regional_tension_points: Optional[List[TensionPointResponse]] = Field(default=None, description="Report related tension points")
    related_reports: Optional[List[ReportContent]] = Field(default=None, description="List of related reports connected to this report")
    released_ts: StrictStr = Field(description="Timestamp when the report was published last time")
    significant_activity: Optional[SignificantActivity] = None
    sources: Optional[List[SourcesResponse]] = Field(default=None, description="List of sources referenced in the report")
    sub_type: Optional[GeopolReportSubType] = None
    title: StrictStr = Field(description="Title of the report")
    type: StrictStr = Field(description="Type of the report")
    victims: Optional[List[ReportsVictimResponse]] = Field(default=None, description="List of purported victims mentioned in the report")
    __properties: ClassVar[List[str]] = ["active_event", "attachments", "body", "classification", "country_profiles", "creation_ts", "derived_entities", "entities", "id", "information_ts", "intelligence_estimate", "is_sensitive_source", "is_truncated", "last_updated_ts", "links", "locations", "regional_tension_points", "related_reports", "released_ts", "significant_activity", "sources", "sub_type", "title", "type", "victims"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_stix(self, settings: Optional["STIXMapperSettings"] = None):
        try:
            from verity471.verity_stix.mappers.common import StixMapper
        except ImportError as e:
            raise ImportError(
                "STIX support is an optional feature. "
                "Install with: pip install 'verity471[stix]'"
            ) from e
        stix_mapper = StixMapper(settings)
        return stix_mapper.map(self.to_dict())

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of GeopolReportDetailsResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in attachments (list)
        _items = []
        if self.attachments:
            for _item_attachments in self.attachments:
                if _item_attachments:
                    _items.append(_item_attachments.to_dict())
            _dict['attachments'] = _items
        # override the default output from pydantic by calling `to_dict()` of classification
        if self.classification:
            _dict['classification'] = self.classification.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in country_profiles (list)
        _items = []
        if self.country_profiles:
            for _item_country_profiles in self.country_profiles:
                if _item_country_profiles:
                    _items.append(_item_country_profiles.to_dict())
            _dict['country_profiles'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in derived_entities (list)
        _items = []
        if self.derived_entities:
            for _item_derived_entities in self.derived_entities:
                if _item_derived_entities:
                    _items.append(_item_derived_entities.to_dict())
            _dict['derived_entities'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in entities (list)
        _items = []
        if self.entities:
            for _item_entities in self.entities:
                if _item_entities:
                    _items.append(_item_entities.to_dict())
            _dict['entities'] = _items
        # override the default output from pydantic by calling `to_dict()` of intelligence_estimate
        if self.intelligence_estimate:
            _dict['intelligence_estimate'] = self.intelligence_estimate.to_dict()
        # override the default output from pydantic by calling `to_dict()` of links
        if self.links:
            _dict['links'] = self.links.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in locations (list)
        _items = []
        if self.locations:
            for _item_locations in self.locations:
                if _item_locations:
                    _items.append(_item_locations.to_dict())
            _dict['locations'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in regional_tension_points (list)
        _items = []
        if self.regional_tension_points:
            for _item_regional_tension_points in self.regional_tension_points:
                if _item_regional_tension_points:
                    _items.append(_item_regional_tension_points.to_dict())
            _dict['regional_tension_points'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in related_reports (list)
        _items = []
        if self.related_reports:
            for _item_related_reports in self.related_reports:
                if _item_related_reports:
                    _items.append(_item_related_reports.to_dict())
            _dict['related_reports'] = _items
        # override the default output from pydantic by calling `to_dict()` of significant_activity
        if self.significant_activity:
            _dict['significant_activity'] = self.significant_activity.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in sources (list)
        _items = []
        if self.sources:
            for _item_sources in self.sources:
                if _item_sources:
                    _items.append(_item_sources.to_dict())
            _dict['sources'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in victims (list)
        _items = []
        if self.victims:
            for _item_victims in self.victims:
                if _item_victims:
                    _items.append(_item_victims.to_dict())
            _dict['victims'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GeopolReportDetailsResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "active_event": obj.get("active_event"),
            "attachments": [ReportAttachment.from_dict(_item) for _item in obj["attachments"]] if obj.get("attachments") is not None else None,
            "body": obj.get("body"),
            "classification": Classification.from_dict(obj["classification"]) if obj.get("classification") is not None else None,
            "country_profiles": [CountryProfileResponse.from_dict(_item) for _item in obj["country_profiles"]] if obj.get("country_profiles") is not None else None,
            "creation_ts": obj.get("creation_ts"),
            "derived_entities": [Entities.from_dict(_item) for _item in obj["derived_entities"]] if obj.get("derived_entities") is not None else None,
            "entities": [Entities.from_dict(_item) for _item in obj["entities"]] if obj.get("entities") is not None else None,
            "id": obj.get("id"),
            "information_ts": obj.get("information_ts"),
            "intelligence_estimate": IntelligenceEstimateResponse.from_dict(obj["intelligence_estimate"]) if obj.get("intelligence_estimate") is not None else None,
            "is_sensitive_source": obj.get("is_sensitive_source"),
            "is_truncated": obj.get("is_truncated"),
            "last_updated_ts": obj.get("last_updated_ts"),
            "links": Links.from_dict(obj["links"]) if obj.get("links") is not None else None,
            "locations": [ReportLocation.from_dict(_item) for _item in obj["locations"]] if obj.get("locations") is not None else None,
            "regional_tension_points": [TensionPointResponse.from_dict(_item) for _item in obj["regional_tension_points"]] if obj.get("regional_tension_points") is not None else None,
            "related_reports": [ReportContent.from_dict(_item) for _item in obj["related_reports"]] if obj.get("related_reports") is not None else None,
            "released_ts": obj.get("released_ts"),
            "significant_activity": SignificantActivity.from_dict(obj["significant_activity"]) if obj.get("significant_activity") is not None else None,
            "sources": [SourcesResponse.from_dict(_item) for _item in obj["sources"]] if obj.get("sources") is not None else None,
            "sub_type": obj.get("sub_type"),
            "title": obj.get("title"),
            "type": obj.get("type"),
            "victims": [ReportsVictimResponse.from_dict(_item) for _item in obj["victims"]] if obj.get("victims") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Verity471 Unified API

      - creds: 1.0.1
  - indicators: 1.0.0
  - malware: 1.0.0
  - reports: 1.0.0
  - sources: 1.0.2
  - actors: 1.0.1
  - watchers: 1.0.0
  - observables: 1.0.0
  - entities: 1.0.0
  - girs: 1.0.0


    The version of the OpenAPI document: 1.1.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictInt, StrictStr, field_validator
from typing import Optional
from typing_extensions import Annotated
from verity471.models.get_cred_occurrence_response import GetCredOccurrenceResponse
from verity471.models.get_cred_occurrence_response_stream import GetCredOccurrenceResponseStream
from verity471.models.get_cred_response import GetCredResponse
from verity471.models.get_cred_response_stream import GetCredResponseStream
from verity471.models.get_cred_set_accessed_url_response_stream import GetCredSetAccessedUrlResponseStream
from verity471.models.get_cred_set_response import GetCredSetResponse
from verity471.models.get_cred_set_response_stream import GetCredSetResponseStream

from verity471.api_client import ApiClient, RequestSerialized
from verity471.api_response import ApiResponse
from verity471.rest import RESTResponseType


class CredentialsApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def get_credential_sets_accessed_urls_stream(
        self,
        credential_set_id: Annotated[Optional[StrictStr], Field(description="Search by credential set id.")] = None,
        credential_set_name: Annotated[Optional[StrictStr], Field(description="Search by credential set name.")] = None,
        accessed_url: Optional[StrictStr] = None,
        girs: Annotated[Optional[StrictStr], Field(description="Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list")] = None,
        victim: Optional[StrictStr] = None,
        var_from: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        until: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        last_updated_from: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        last_updated_until: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        size: Annotated[Optional[StrictInt], Field(description="Range is: [1, 1000]")] = None,
        cursor: Annotated[Optional[StrictStr], Field(description="Continue scrolling from cursor")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetCredSetAccessedUrlResponseStream:
        """Credential set accessed url stream

        Returns list of Credential sets accessed urls matching filter criteria. The response contains “cursor_next” field, which should be provided to the next subsequent call to fetch potential next page of the results. Results are sorted by ascending order of the last_updated_ts field. Two indications of the end of the stream are possible: the result list is empty or its size is less than the requested items count.

        :param credential_set_id: Search by credential set id.
        :type credential_set_id: str
        :param credential_set_name: Search by credential set name.
        :type credential_set_name: str
        :param accessed_url:
        :type accessed_url: str
        :param girs: Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list
        :type girs: str
        :param victim:
        :type victim: str
        :param var_from: UNIX timestamp(in milliseconds)
        :type var_from: int
        :param until: UNIX timestamp(in milliseconds)
        :type until: int
        :param last_updated_from: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_from: str
        :param last_updated_until: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_until: str
        :param size: Range is: [1, 1000]
        :type size: int
        :param cursor: Continue scrolling from cursor
        :type cursor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credential_sets_accessed_urls_stream_serialize(
            credential_set_id=credential_set_id,
            credential_set_name=credential_set_name,
            accessed_url=accessed_url,
            girs=girs,
            victim=victim,
            var_from=var_from,
            until=until,
            last_updated_from=last_updated_from,
            last_updated_until=last_updated_until,
            size=size,
            cursor=cursor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredSetAccessedUrlResponseStream",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_credential_sets_accessed_urls_stream_with_http_info(
        self,
        credential_set_id: Annotated[Optional[StrictStr], Field(description="Search by credential set id.")] = None,
        credential_set_name: Annotated[Optional[StrictStr], Field(description="Search by credential set name.")] = None,
        accessed_url: Optional[StrictStr] = None,
        girs: Annotated[Optional[StrictStr], Field(description="Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list")] = None,
        victim: Optional[StrictStr] = None,
        var_from: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        until: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        last_updated_from: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        last_updated_until: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        size: Annotated[Optional[StrictInt], Field(description="Range is: [1, 1000]")] = None,
        cursor: Annotated[Optional[StrictStr], Field(description="Continue scrolling from cursor")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetCredSetAccessedUrlResponseStream]:
        """Credential set accessed url stream

        Returns list of Credential sets accessed urls matching filter criteria. The response contains “cursor_next” field, which should be provided to the next subsequent call to fetch potential next page of the results. Results are sorted by ascending order of the last_updated_ts field. Two indications of the end of the stream are possible: the result list is empty or its size is less than the requested items count.

        :param credential_set_id: Search by credential set id.
        :type credential_set_id: str
        :param credential_set_name: Search by credential set name.
        :type credential_set_name: str
        :param accessed_url:
        :type accessed_url: str
        :param girs: Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list
        :type girs: str
        :param victim:
        :type victim: str
        :param var_from: UNIX timestamp(in milliseconds)
        :type var_from: int
        :param until: UNIX timestamp(in milliseconds)
        :type until: int
        :param last_updated_from: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_from: str
        :param last_updated_until: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_until: str
        :param size: Range is: [1, 1000]
        :type size: int
        :param cursor: Continue scrolling from cursor
        :type cursor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credential_sets_accessed_urls_stream_serialize(
            credential_set_id=credential_set_id,
            credential_set_name=credential_set_name,
            accessed_url=accessed_url,
            girs=girs,
            victim=victim,
            var_from=var_from,
            until=until,
            last_updated_from=last_updated_from,
            last_updated_until=last_updated_until,
            size=size,
            cursor=cursor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredSetAccessedUrlResponseStream",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_credential_sets_accessed_urls_stream_without_preload_content(
        self,
        credential_set_id: Annotated[Optional[StrictStr], Field(description="Search by credential set id.")] = None,
        credential_set_name: Annotated[Optional[StrictStr], Field(description="Search by credential set name.")] = None,
        accessed_url: Optional[StrictStr] = None,
        girs: Annotated[Optional[StrictStr], Field(description="Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list")] = None,
        victim: Optional[StrictStr] = None,
        var_from: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        until: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        last_updated_from: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        last_updated_until: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        size: Annotated[Optional[StrictInt], Field(description="Range is: [1, 1000]")] = None,
        cursor: Annotated[Optional[StrictStr], Field(description="Continue scrolling from cursor")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Credential set accessed url stream

        Returns list of Credential sets accessed urls matching filter criteria. The response contains “cursor_next” field, which should be provided to the next subsequent call to fetch potential next page of the results. Results are sorted by ascending order of the last_updated_ts field. Two indications of the end of the stream are possible: the result list is empty or its size is less than the requested items count.

        :param credential_set_id: Search by credential set id.
        :type credential_set_id: str
        :param credential_set_name: Search by credential set name.
        :type credential_set_name: str
        :param accessed_url:
        :type accessed_url: str
        :param girs: Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list
        :type girs: str
        :param victim:
        :type victim: str
        :param var_from: UNIX timestamp(in milliseconds)
        :type var_from: int
        :param until: UNIX timestamp(in milliseconds)
        :type until: int
        :param last_updated_from: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_from: str
        :param last_updated_until: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_until: str
        :param size: Range is: [1, 1000]
        :type size: int
        :param cursor: Continue scrolling from cursor
        :type cursor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credential_sets_accessed_urls_stream_serialize(
            credential_set_id=credential_set_id,
            credential_set_name=credential_set_name,
            accessed_url=accessed_url,
            girs=girs,
            victim=victim,
            var_from=var_from,
            until=until,
            last_updated_from=last_updated_from,
            last_updated_until=last_updated_until,
            size=size,
            cursor=cursor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredSetAccessedUrlResponseStream",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_credential_sets_accessed_urls_stream_serialize(
        self,
        credential_set_id,
        credential_set_name,
        accessed_url,
        girs,
        victim,
        var_from,
        until,
        last_updated_from,
        last_updated_until,
        size,
        cursor,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if credential_set_id is not None:
            
            _query_params.append(('credential_set_id', credential_set_id))
            
        if credential_set_name is not None:
            
            _query_params.append(('credential_set_name', credential_set_name))
            
        if accessed_url is not None:
            
            _query_params.append(('accessed_url', accessed_url))
            
        if girs is not None:
            
            _query_params.append(('girs', girs))
            
        if victim is not None:
            
            _query_params.append(('victim', victim))
            
        if var_from is not None:
            
            _query_params.append(('from', var_from))
            
        if until is not None:
            
            _query_params.append(('until', until))
            
        if last_updated_from is not None:
            
            _query_params.append(('last_updated_from', last_updated_from))
            
        if last_updated_until is not None:
            
            _query_params.append(('last_updated_until', last_updated_until))
            
        if size is not None:
            
            _query_params.append(('size', size))
            
        if cursor is not None:
            
            _query_params.append(('cursor', cursor))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/integrations/creds/v1/credential-sets/accessed-urls/stream',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_credential_sets_id(
        self,
        id: Annotated[StrictStr, Field(description="ID of the credential set")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetCredSetResponse:
        """Get credential set by ID

        Returns a single Credential set matching requested ID.

        :param id: ID of the credential set (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credential_sets_id_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredSetResponse",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_credential_sets_id_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="ID of the credential set")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetCredSetResponse]:
        """Get credential set by ID

        Returns a single Credential set matching requested ID.

        :param id: ID of the credential set (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credential_sets_id_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredSetResponse",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_credential_sets_id_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="ID of the credential set")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get credential set by ID

        Returns a single Credential set matching requested ID.

        :param id: ID of the credential set (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credential_sets_id_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredSetResponse",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_credential_sets_id_serialize(
        self,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/integrations/creds/v1/credential-sets/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_credential_sets_stream(
        self,
        credential_set_name: Annotated[Optional[StrictStr], Field(description="Search by credential set name.")] = None,
        girs: Annotated[Optional[StrictStr], Field(description="Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list")] = None,
        victim: Optional[StrictStr] = None,
        var_from: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        until: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        last_updated_from: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        last_updated_until: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        size: Annotated[Optional[StrictInt], Field(description="Range is: [1, 1000]")] = None,
        cursor: Annotated[Optional[StrictStr], Field(description="Continue scrolling from cursor")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetCredSetResponseStream:
        """Credential set stream

        Returns list of Credential sets matching filter criteria. The response contains “cursor_next” field, which should be provided to the next subsequent call to fetch potential next page of the results. Results are sorted by ascending order of the last_updated_ts field. Two indications of the end of the stream are possible: the result list is empty or its size is less than the requested items count.

        :param credential_set_name: Search by credential set name.
        :type credential_set_name: str
        :param girs: Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list
        :type girs: str
        :param victim:
        :type victim: str
        :param var_from: UNIX timestamp(in milliseconds)
        :type var_from: int
        :param until: UNIX timestamp(in milliseconds)
        :type until: int
        :param last_updated_from: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_from: str
        :param last_updated_until: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_until: str
        :param size: Range is: [1, 1000]
        :type size: int
        :param cursor: Continue scrolling from cursor
        :type cursor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credential_sets_stream_serialize(
            credential_set_name=credential_set_name,
            girs=girs,
            victim=victim,
            var_from=var_from,
            until=until,
            last_updated_from=last_updated_from,
            last_updated_until=last_updated_until,
            size=size,
            cursor=cursor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredSetResponseStream",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_credential_sets_stream_with_http_info(
        self,
        credential_set_name: Annotated[Optional[StrictStr], Field(description="Search by credential set name.")] = None,
        girs: Annotated[Optional[StrictStr], Field(description="Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list")] = None,
        victim: Optional[StrictStr] = None,
        var_from: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        until: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        last_updated_from: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        last_updated_until: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        size: Annotated[Optional[StrictInt], Field(description="Range is: [1, 1000]")] = None,
        cursor: Annotated[Optional[StrictStr], Field(description="Continue scrolling from cursor")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetCredSetResponseStream]:
        """Credential set stream

        Returns list of Credential sets matching filter criteria. The response contains “cursor_next” field, which should be provided to the next subsequent call to fetch potential next page of the results. Results are sorted by ascending order of the last_updated_ts field. Two indications of the end of the stream are possible: the result list is empty or its size is less than the requested items count.

        :param credential_set_name: Search by credential set name.
        :type credential_set_name: str
        :param girs: Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list
        :type girs: str
        :param victim:
        :type victim: str
        :param var_from: UNIX timestamp(in milliseconds)
        :type var_from: int
        :param until: UNIX timestamp(in milliseconds)
        :type until: int
        :param last_updated_from: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_from: str
        :param last_updated_until: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_until: str
        :param size: Range is: [1, 1000]
        :type size: int
        :param cursor: Continue scrolling from cursor
        :type cursor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credential_sets_stream_serialize(
            credential_set_name=credential_set_name,
            girs=girs,
            victim=victim,
            var_from=var_from,
            until=until,
            last_updated_from=last_updated_from,
            last_updated_until=last_updated_until,
            size=size,
            cursor=cursor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredSetResponseStream",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_credential_sets_stream_without_preload_content(
        self,
        credential_set_name: Annotated[Optional[StrictStr], Field(description="Search by credential set name.")] = None,
        girs: Annotated[Optional[StrictStr], Field(description="Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list")] = None,
        victim: Optional[StrictStr] = None,
        var_from: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        until: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        last_updated_from: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        last_updated_until: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        size: Annotated[Optional[StrictInt], Field(description="Range is: [1, 1000]")] = None,
        cursor: Annotated[Optional[StrictStr], Field(description="Continue scrolling from cursor")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Credential set stream

        Returns list of Credential sets matching filter criteria. The response contains “cursor_next” field, which should be provided to the next subsequent call to fetch potential next page of the results. Results are sorted by ascending order of the last_updated_ts field. Two indications of the end of the stream are possible: the result list is empty or its size is less than the requested items count.

        :param credential_set_name: Search by credential set name.
        :type credential_set_name: str
        :param girs: Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list
        :type girs: str
        :param victim:
        :type victim: str
        :param var_from: UNIX timestamp(in milliseconds)
        :type var_from: int
        :param until: UNIX timestamp(in milliseconds)
        :type until: int
        :param last_updated_from: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_from: str
        :param last_updated_until: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_until: str
        :param size: Range is: [1, 1000]
        :type size: int
        :param cursor: Continue scrolling from cursor
        :type cursor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credential_sets_stream_serialize(
            credential_set_name=credential_set_name,
            girs=girs,
            victim=victim,
            var_from=var_from,
            until=until,
            last_updated_from=last_updated_from,
            last_updated_until=last_updated_until,
            size=size,
            cursor=cursor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredSetResponseStream",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_credential_sets_stream_serialize(
        self,
        credential_set_name,
        girs,
        victim,
        var_from,
        until,
        last_updated_from,
        last_updated_until,
        size,
        cursor,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if credential_set_name is not None:
            
            _query_params.append(('credential_set_name', credential_set_name))
            
        if girs is not None:
            
            _query_params.append(('girs', girs))
            
        if victim is not None:
            
            _query_params.append(('victim', victim))
            
        if var_from is not None:
            
            _query_params.append(('from', var_from))
            
        if until is not None:
            
            _query_params.append(('until', until))
            
        if last_updated_from is not None:
            
            _query_params.append(('last_updated_from', last_updated_from))
            
        if last_updated_until is not None:
            
            _query_params.append(('last_updated_until', last_updated_until))
            
        if size is not None:
            
            _query_params.append(('size', size))
            
        if cursor is not None:
            
            _query_params.append(('cursor', cursor))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/integrations/creds/v1/credential-sets/stream',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_credentials_id(
        self,
        id: Annotated[StrictStr, Field(description="ID of the credential")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetCredResponse:
        """Get credential by ID

        Returns a single Credential matching requested ID.

        :param id: ID of the credential (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credentials_id_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredResponse",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_credentials_id_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="ID of the credential")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetCredResponse]:
        """Get credential by ID

        Returns a single Credential matching requested ID.

        :param id: ID of the credential (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credentials_id_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredResponse",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_credentials_id_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="ID of the credential")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get credential by ID

        Returns a single Credential matching requested ID.

        :param id: ID of the credential (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credentials_id_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredResponse",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_credentials_id_serialize(
        self,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/integrations/creds/v1/credentials/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_credentials_occurrences_id(
        self,
        id: Annotated[StrictStr, Field(description="ID of the credential occurrence")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetCredOccurrenceResponse:
        """Get credential occurrence by ID

        Returns a single Credential occurrence matching requested ID.

        :param id: ID of the credential occurrence (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credentials_occurrences_id_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredOccurrenceResponse",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_credentials_occurrences_id_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="ID of the credential occurrence")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetCredOccurrenceResponse]:
        """Get credential occurrence by ID

        Returns a single Credential occurrence matching requested ID.

        :param id: ID of the credential occurrence (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credentials_occurrences_id_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredOccurrenceResponse",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_credentials_occurrences_id_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="ID of the credential occurrence")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get credential occurrence by ID

        Returns a single Credential occurrence matching requested ID.

        :param id: ID of the credential occurrence (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credentials_occurrences_id_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredOccurrenceResponse",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_credentials_occurrences_id_serialize(
        self,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/integrations/creds/v1/credentials/occurrences/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_credentials_occurrences_stream(
        self,
        credential_id: Annotated[Optional[StrictStr], Field(description="Search by credential id.")] = None,
        credential_set_name: Annotated[Optional[StrictStr], Field(description="Search by credential set name.")] = None,
        credential_set_id: Annotated[Optional[StrictStr], Field(description="Search by credential set id.")] = None,
        domain: Annotated[Optional[StrictStr], Field(description="Search by credential domain(detection domain).")] = None,
        affiliation_group: Annotated[Optional[StrictStr], Field(description="Possible values: my_employees, my_customers, third_parties, vip_emails")] = None,
        password_strength: Annotated[Optional[StrictStr], Field(description="Possible values: excellent, strong, medium, weak, poor, not_provided")] = None,
        password_length_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_lowercase_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_uppercase_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_numbers_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_punctuation_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_symbols_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_separators_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_other_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_entropy_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_plain: Annotated[Optional[StrictStr], Field(description="Must be URL encoded")] = None,
        credential_login: Annotated[Optional[StrictStr], Field(description="Search by credential login.")] = None,
        detected_malware: Annotated[Optional[StrictStr], Field(description="Search by credential detected malware. For example: agent_tesla, Lumma, VIDAR")] = None,
        girs: Annotated[Optional[StrictStr], Field(description="Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list")] = None,
        var_from: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        until: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        last_updated_from: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        last_updated_until: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        size: Annotated[Optional[StrictInt], Field(description="Range is: [1, 1000]")] = None,
        cursor: Annotated[Optional[StrictStr], Field(description="Continue scrolling from cursor")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetCredOccurrenceResponseStream:
        """Credential occurrence stream

        Returns list of Credential occurrences matching filter criteria. Stream pagination is based on a cursor. The response is the same as for the /credentials/occurrences endpoint but with the additional “cursorNext” field. Results are sorted by ascending order of the lastUpdated field. Two indications of the end of the stream are possible: the result list is empty or its size is less than the requested items count.

        :param credential_id: Search by credential id.
        :type credential_id: str
        :param credential_set_name: Search by credential set name.
        :type credential_set_name: str
        :param credential_set_id: Search by credential set id.
        :type credential_set_id: str
        :param domain: Search by credential domain(detection domain).
        :type domain: str
        :param affiliation_group: Possible values: my_employees, my_customers, third_parties, vip_emails
        :type affiliation_group: str
        :param password_strength: Possible values: excellent, strong, medium, weak, poor, not_provided
        :type password_strength: str
        :param password_length_gte: Must be greater than or equal to 0
        :type password_length_gte: int
        :param password_lowercase_gte: Must be greater than or equal to 0
        :type password_lowercase_gte: int
        :param password_uppercase_gte: Must be greater than or equal to 0
        :type password_uppercase_gte: int
        :param password_numbers_gte: Must be greater than or equal to 0
        :type password_numbers_gte: int
        :param password_punctuation_gte: Must be greater than or equal to 0
        :type password_punctuation_gte: int
        :param password_symbols_gte: Must be greater than or equal to 0
        :type password_symbols_gte: int
        :param password_separators_gte: Must be greater than or equal to 0
        :type password_separators_gte: int
        :param password_other_gte: Must be greater than or equal to 0
        :type password_other_gte: int
        :param password_entropy_gte: Must be greater than or equal to 0
        :type password_entropy_gte: int
        :param password_plain: Must be URL encoded
        :type password_plain: str
        :param credential_login: Search by credential login.
        :type credential_login: str
        :param detected_malware: Search by credential detected malware. For example: agent_tesla, Lumma, VIDAR
        :type detected_malware: str
        :param girs: Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list
        :type girs: str
        :param var_from: UNIX timestamp(in milliseconds)
        :type var_from: int
        :param until: UNIX timestamp(in milliseconds)
        :type until: int
        :param last_updated_from: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_from: str
        :param last_updated_until: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_until: str
        :param size: Range is: [1, 1000]
        :type size: int
        :param cursor: Continue scrolling from cursor
        :type cursor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credentials_occurrences_stream_serialize(
            credential_id=credential_id,
            credential_set_name=credential_set_name,
            credential_set_id=credential_set_id,
            domain=domain,
            affiliation_group=affiliation_group,
            password_strength=password_strength,
            password_length_gte=password_length_gte,
            password_lowercase_gte=password_lowercase_gte,
            password_uppercase_gte=password_uppercase_gte,
            password_numbers_gte=password_numbers_gte,
            password_punctuation_gte=password_punctuation_gte,
            password_symbols_gte=password_symbols_gte,
            password_separators_gte=password_separators_gte,
            password_other_gte=password_other_gte,
            password_entropy_gte=password_entropy_gte,
            password_plain=password_plain,
            credential_login=credential_login,
            detected_malware=detected_malware,
            girs=girs,
            var_from=var_from,
            until=until,
            last_updated_from=last_updated_from,
            last_updated_until=last_updated_until,
            size=size,
            cursor=cursor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredOccurrenceResponseStream",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_credentials_occurrences_stream_with_http_info(
        self,
        credential_id: Annotated[Optional[StrictStr], Field(description="Search by credential id.")] = None,
        credential_set_name: Annotated[Optional[StrictStr], Field(description="Search by credential set name.")] = None,
        credential_set_id: Annotated[Optional[StrictStr], Field(description="Search by credential set id.")] = None,
        domain: Annotated[Optional[StrictStr], Field(description="Search by credential domain(detection domain).")] = None,
        affiliation_group: Annotated[Optional[StrictStr], Field(description="Possible values: my_employees, my_customers, third_parties, vip_emails")] = None,
        password_strength: Annotated[Optional[StrictStr], Field(description="Possible values: excellent, strong, medium, weak, poor, not_provided")] = None,
        password_length_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_lowercase_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_uppercase_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_numbers_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_punctuation_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_symbols_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_separators_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_other_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_entropy_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_plain: Annotated[Optional[StrictStr], Field(description="Must be URL encoded")] = None,
        credential_login: Annotated[Optional[StrictStr], Field(description="Search by credential login.")] = None,
        detected_malware: Annotated[Optional[StrictStr], Field(description="Search by credential detected malware. For example: agent_tesla, Lumma, VIDAR")] = None,
        girs: Annotated[Optional[StrictStr], Field(description="Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list")] = None,
        var_from: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        until: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        last_updated_from: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        last_updated_until: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        size: Annotated[Optional[StrictInt], Field(description="Range is: [1, 1000]")] = None,
        cursor: Annotated[Optional[StrictStr], Field(description="Continue scrolling from cursor")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetCredOccurrenceResponseStream]:
        """Credential occurrence stream

        Returns list of Credential occurrences matching filter criteria. Stream pagination is based on a cursor. The response is the same as for the /credentials/occurrences endpoint but with the additional “cursorNext” field. Results are sorted by ascending order of the lastUpdated field. Two indications of the end of the stream are possible: the result list is empty or its size is less than the requested items count.

        :param credential_id: Search by credential id.
        :type credential_id: str
        :param credential_set_name: Search by credential set name.
        :type credential_set_name: str
        :param credential_set_id: Search by credential set id.
        :type credential_set_id: str
        :param domain: Search by credential domain(detection domain).
        :type domain: str
        :param affiliation_group: Possible values: my_employees, my_customers, third_parties, vip_emails
        :type affiliation_group: str
        :param password_strength: Possible values: excellent, strong, medium, weak, poor, not_provided
        :type password_strength: str
        :param password_length_gte: Must be greater than or equal to 0
        :type password_length_gte: int
        :param password_lowercase_gte: Must be greater than or equal to 0
        :type password_lowercase_gte: int
        :param password_uppercase_gte: Must be greater than or equal to 0
        :type password_uppercase_gte: int
        :param password_numbers_gte: Must be greater than or equal to 0
        :type password_numbers_gte: int
        :param password_punctuation_gte: Must be greater than or equal to 0
        :type password_punctuation_gte: int
        :param password_symbols_gte: Must be greater than or equal to 0
        :type password_symbols_gte: int
        :param password_separators_gte: Must be greater than or equal to 0
        :type password_separators_gte: int
        :param password_other_gte: Must be greater than or equal to 0
        :type password_other_gte: int
        :param password_entropy_gte: Must be greater than or equal to 0
        :type password_entropy_gte: int
        :param password_plain: Must be URL encoded
        :type password_plain: str
        :param credential_login: Search by credential login.
        :type credential_login: str
        :param detected_malware: Search by credential detected malware. For example: agent_tesla, Lumma, VIDAR
        :type detected_malware: str
        :param girs: Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list
        :type girs: str
        :param var_from: UNIX timestamp(in milliseconds)
        :type var_from: int
        :param until: UNIX timestamp(in milliseconds)
        :type until: int
        :param last_updated_from: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_from: str
        :param last_updated_until: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_until: str
        :param size: Range is: [1, 1000]
        :type size: int
        :param cursor: Continue scrolling from cursor
        :type cursor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credentials_occurrences_stream_serialize(
            credential_id=credential_id,
            credential_set_name=credential_set_name,
            credential_set_id=credential_set_id,
            domain=domain,
            affiliation_group=affiliation_group,
            password_strength=password_strength,
            password_length_gte=password_length_gte,
            password_lowercase_gte=password_lowercase_gte,
            password_uppercase_gte=password_uppercase_gte,
            password_numbers_gte=password_numbers_gte,
            password_punctuation_gte=password_punctuation_gte,
            password_symbols_gte=password_symbols_gte,
            password_separators_gte=password_separators_gte,
            password_other_gte=password_other_gte,
            password_entropy_gte=password_entropy_gte,
            password_plain=password_plain,
            credential_login=credential_login,
            detected_malware=detected_malware,
            girs=girs,
            var_from=var_from,
            until=until,
            last_updated_from=last_updated_from,
            last_updated_until=last_updated_until,
            size=size,
            cursor=cursor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredOccurrenceResponseStream",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_credentials_occurrences_stream_without_preload_content(
        self,
        credential_id: Annotated[Optional[StrictStr], Field(description="Search by credential id.")] = None,
        credential_set_name: Annotated[Optional[StrictStr], Field(description="Search by credential set name.")] = None,
        credential_set_id: Annotated[Optional[StrictStr], Field(description="Search by credential set id.")] = None,
        domain: Annotated[Optional[StrictStr], Field(description="Search by credential domain(detection domain).")] = None,
        affiliation_group: Annotated[Optional[StrictStr], Field(description="Possible values: my_employees, my_customers, third_parties, vip_emails")] = None,
        password_strength: Annotated[Optional[StrictStr], Field(description="Possible values: excellent, strong, medium, weak, poor, not_provided")] = None,
        password_length_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_lowercase_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_uppercase_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_numbers_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_punctuation_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_symbols_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_separators_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_other_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_entropy_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_plain: Annotated[Optional[StrictStr], Field(description="Must be URL encoded")] = None,
        credential_login: Annotated[Optional[StrictStr], Field(description="Search by credential login.")] = None,
        detected_malware: Annotated[Optional[StrictStr], Field(description="Search by credential detected malware. For example: agent_tesla, Lumma, VIDAR")] = None,
        girs: Annotated[Optional[StrictStr], Field(description="Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list")] = None,
        var_from: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        until: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        last_updated_from: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        last_updated_until: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        size: Annotated[Optional[StrictInt], Field(description="Range is: [1, 1000]")] = None,
        cursor: Annotated[Optional[StrictStr], Field(description="Continue scrolling from cursor")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Credential occurrence stream

        Returns list of Credential occurrences matching filter criteria. Stream pagination is based on a cursor. The response is the same as for the /credentials/occurrences endpoint but with the additional “cursorNext” field. Results are sorted by ascending order of the lastUpdated field. Two indications of the end of the stream are possible: the result list is empty or its size is less than the requested items count.

        :param credential_id: Search by credential id.
        :type credential_id: str
        :param credential_set_name: Search by credential set name.
        :type credential_set_name: str
        :param credential_set_id: Search by credential set id.
        :type credential_set_id: str
        :param domain: Search by credential domain(detection domain).
        :type domain: str
        :param affiliation_group: Possible values: my_employees, my_customers, third_parties, vip_emails
        :type affiliation_group: str
        :param password_strength: Possible values: excellent, strong, medium, weak, poor, not_provided
        :type password_strength: str
        :param password_length_gte: Must be greater than or equal to 0
        :type password_length_gte: int
        :param password_lowercase_gte: Must be greater than or equal to 0
        :type password_lowercase_gte: int
        :param password_uppercase_gte: Must be greater than or equal to 0
        :type password_uppercase_gte: int
        :param password_numbers_gte: Must be greater than or equal to 0
        :type password_numbers_gte: int
        :param password_punctuation_gte: Must be greater than or equal to 0
        :type password_punctuation_gte: int
        :param password_symbols_gte: Must be greater than or equal to 0
        :type password_symbols_gte: int
        :param password_separators_gte: Must be greater than or equal to 0
        :type password_separators_gte: int
        :param password_other_gte: Must be greater than or equal to 0
        :type password_other_gte: int
        :param password_entropy_gte: Must be greater than or equal to 0
        :type password_entropy_gte: int
        :param password_plain: Must be URL encoded
        :type password_plain: str
        :param credential_login: Search by credential login.
        :type credential_login: str
        :param detected_malware: Search by credential detected malware. For example: agent_tesla, Lumma, VIDAR
        :type detected_malware: str
        :param girs: Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list
        :type girs: str
        :param var_from: UNIX timestamp(in milliseconds)
        :type var_from: int
        :param until: UNIX timestamp(in milliseconds)
        :type until: int
        :param last_updated_from: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_from: str
        :param last_updated_until: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_until: str
        :param size: Range is: [1, 1000]
        :type size: int
        :param cursor: Continue scrolling from cursor
        :type cursor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credentials_occurrences_stream_serialize(
            credential_id=credential_id,
            credential_set_name=credential_set_name,
            credential_set_id=credential_set_id,
            domain=domain,
            affiliation_group=affiliation_group,
            password_strength=password_strength,
            password_length_gte=password_length_gte,
            password_lowercase_gte=password_lowercase_gte,
            password_uppercase_gte=password_uppercase_gte,
            password_numbers_gte=password_numbers_gte,
            password_punctuation_gte=password_punctuation_gte,
            password_symbols_gte=password_symbols_gte,
            password_separators_gte=password_separators_gte,
            password_other_gte=password_other_gte,
            password_entropy_gte=password_entropy_gte,
            password_plain=password_plain,
            credential_login=credential_login,
            detected_malware=detected_malware,
            girs=girs,
            var_from=var_from,
            until=until,
            last_updated_from=last_updated_from,
            last_updated_until=last_updated_until,
            size=size,
            cursor=cursor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredOccurrenceResponseStream",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_credentials_occurrences_stream_serialize(
        self,
        credential_id,
        credential_set_name,
        credential_set_id,
        domain,
        affiliation_group,
        password_strength,
        password_length_gte,
        password_lowercase_gte,
        password_uppercase_gte,
        password_numbers_gte,
        password_punctuation_gte,
        password_symbols_gte,
        password_separators_gte,
        password_other_gte,
        password_entropy_gte,
        password_plain,
        credential_login,
        detected_malware,
        girs,
        var_from,
        until,
        last_updated_from,
        last_updated_until,
        size,
        cursor,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if credential_id is not None:
            
            _query_params.append(('credential_id', credential_id))
            
        if credential_set_name is not None:
            
            _query_params.append(('credential_set_name', credential_set_name))
            
        if credential_set_id is not None:
            
            _query_params.append(('credential_set_id', credential_set_id))
            
        if domain is not None:
            
            _query_params.append(('domain', domain))
            
        if affiliation_group is not None:
            
            _query_params.append(('affiliation_group', affiliation_group))
            
        if password_strength is not None:
            
            _query_params.append(('password_strength', password_strength))
            
        if password_length_gte is not None:
            
            _query_params.append(('password_length_gte', password_length_gte))
            
        if password_lowercase_gte is not None:
            
            _query_params.append(('password_lowercase_gte', password_lowercase_gte))
            
        if password_uppercase_gte is not None:
            
            _query_params.append(('password_uppercase_gte', password_uppercase_gte))
            
        if password_numbers_gte is not None:
            
            _query_params.append(('password_numbers_gte', password_numbers_gte))
            
        if password_punctuation_gte is not None:
            
            _query_params.append(('password_punctuation_gte', password_punctuation_gte))
            
        if password_symbols_gte is not None:
            
            _query_params.append(('password_symbols_gte', password_symbols_gte))
            
        if password_separators_gte is not None:
            
            _query_params.append(('password_separators_gte', password_separators_gte))
            
        if password_other_gte is not None:
            
            _query_params.append(('password_other_gte', password_other_gte))
            
        if password_entropy_gte is not None:
            
            _query_params.append(('password_entropy_gte', password_entropy_gte))
            
        if password_plain is not None:
            
            _query_params.append(('password_plain', password_plain))
            
        if credential_login is not None:
            
            _query_params.append(('credential_login', credential_login))
            
        if detected_malware is not None:
            
            _query_params.append(('detected_malware', detected_malware))
            
        if girs is not None:
            
            _query_params.append(('girs', girs))
            
        if var_from is not None:
            
            _query_params.append(('from', var_from))
            
        if until is not None:
            
            _query_params.append(('until', until))
            
        if last_updated_from is not None:
            
            _query_params.append(('last_updated_from', last_updated_from))
            
        if last_updated_until is not None:
            
            _query_params.append(('last_updated_until', last_updated_until))
            
        if size is not None:
            
            _query_params.append(('size', size))
            
        if cursor is not None:
            
            _query_params.append(('cursor', cursor))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/integrations/creds/v1/credentials/occurrences/stream',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_credentials_stream(
        self,
        credential_set_name: Annotated[Optional[StrictStr], Field(description="Search by credential set name.")] = None,
        credential_set_id: Annotated[Optional[StrictStr], Field(description="Search by credential set id.")] = None,
        domain: Annotated[Optional[StrictStr], Field(description="Search by credential domain(detection domain).")] = None,
        affiliation_group: Annotated[Optional[StrictStr], Field(description="Possible values: my_employees, my_customers, third_parties, vip_emails")] = None,
        password_strength: Annotated[Optional[StrictStr], Field(description="Possible values: excellent, strong, medium, weak, poor, not_provided")] = None,
        password_length_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_lowercase_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_uppercase_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_numbers_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_punctuation_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_symbols_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_separators_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_other_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_entropy_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_plain: Annotated[Optional[StrictStr], Field(description="Must be URL encoded")] = None,
        credential_login: Annotated[Optional[StrictStr], Field(description="Search by credential login.")] = None,
        detected_malware: Annotated[Optional[StrictStr], Field(description="Search by credential detected malware. For example: agent_tesla, Lumma, VIDAR")] = None,
        girs: Annotated[Optional[StrictStr], Field(description="Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list")] = None,
        var_from: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        until: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        last_updated_from: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        last_updated_until: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        size: Annotated[Optional[StrictInt], Field(description="Range is: [1, 1000]")] = None,
        cursor: Annotated[Optional[StrictStr], Field(description="Continue scrolling from cursor")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetCredResponseStream:
        """Credential stream

        Returns list of Credentials matching filter criteria. The response contains “cursor_next” field, which should be provided to the next subsequent call to fetch potential next page of the results. Results are sorted by ascending order of the last_updated_ts field. Two indications of the end of the stream are possible: the result list is empty or its size is less than the requested items count.

        :param credential_set_name: Search by credential set name.
        :type credential_set_name: str
        :param credential_set_id: Search by credential set id.
        :type credential_set_id: str
        :param domain: Search by credential domain(detection domain).
        :type domain: str
        :param affiliation_group: Possible values: my_employees, my_customers, third_parties, vip_emails
        :type affiliation_group: str
        :param password_strength: Possible values: excellent, strong, medium, weak, poor, not_provided
        :type password_strength: str
        :param password_length_gte: Must be greater than or equal to 0
        :type password_length_gte: int
        :param password_lowercase_gte: Must be greater than or equal to 0
        :type password_lowercase_gte: int
        :param password_uppercase_gte: Must be greater than or equal to 0
        :type password_uppercase_gte: int
        :param password_numbers_gte: Must be greater than or equal to 0
        :type password_numbers_gte: int
        :param password_punctuation_gte: Must be greater than or equal to 0
        :type password_punctuation_gte: int
        :param password_symbols_gte: Must be greater than or equal to 0
        :type password_symbols_gte: int
        :param password_separators_gte: Must be greater than or equal to 0
        :type password_separators_gte: int
        :param password_other_gte: Must be greater than or equal to 0
        :type password_other_gte: int
        :param password_entropy_gte: Must be greater than or equal to 0
        :type password_entropy_gte: int
        :param password_plain: Must be URL encoded
        :type password_plain: str
        :param credential_login: Search by credential login.
        :type credential_login: str
        :param detected_malware: Search by credential detected malware. For example: agent_tesla, Lumma, VIDAR
        :type detected_malware: str
        :param girs: Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list
        :type girs: str
        :param var_from: UNIX timestamp(in milliseconds)
        :type var_from: int
        :param until: UNIX timestamp(in milliseconds)
        :type until: int
        :param last_updated_from: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_from: str
        :param last_updated_until: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_until: str
        :param size: Range is: [1, 1000]
        :type size: int
        :param cursor: Continue scrolling from cursor
        :type cursor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credentials_stream_serialize(
            credential_set_name=credential_set_name,
            credential_set_id=credential_set_id,
            domain=domain,
            affiliation_group=affiliation_group,
            password_strength=password_strength,
            password_length_gte=password_length_gte,
            password_lowercase_gte=password_lowercase_gte,
            password_uppercase_gte=password_uppercase_gte,
            password_numbers_gte=password_numbers_gte,
            password_punctuation_gte=password_punctuation_gte,
            password_symbols_gte=password_symbols_gte,
            password_separators_gte=password_separators_gte,
            password_other_gte=password_other_gte,
            password_entropy_gte=password_entropy_gte,
            password_plain=password_plain,
            credential_login=credential_login,
            detected_malware=detected_malware,
            girs=girs,
            var_from=var_from,
            until=until,
            last_updated_from=last_updated_from,
            last_updated_until=last_updated_until,
            size=size,
            cursor=cursor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredResponseStream",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_credentials_stream_with_http_info(
        self,
        credential_set_name: Annotated[Optional[StrictStr], Field(description="Search by credential set name.")] = None,
        credential_set_id: Annotated[Optional[StrictStr], Field(description="Search by credential set id.")] = None,
        domain: Annotated[Optional[StrictStr], Field(description="Search by credential domain(detection domain).")] = None,
        affiliation_group: Annotated[Optional[StrictStr], Field(description="Possible values: my_employees, my_customers, third_parties, vip_emails")] = None,
        password_strength: Annotated[Optional[StrictStr], Field(description="Possible values: excellent, strong, medium, weak, poor, not_provided")] = None,
        password_length_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_lowercase_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_uppercase_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_numbers_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_punctuation_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_symbols_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_separators_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_other_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_entropy_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_plain: Annotated[Optional[StrictStr], Field(description="Must be URL encoded")] = None,
        credential_login: Annotated[Optional[StrictStr], Field(description="Search by credential login.")] = None,
        detected_malware: Annotated[Optional[StrictStr], Field(description="Search by credential detected malware. For example: agent_tesla, Lumma, VIDAR")] = None,
        girs: Annotated[Optional[StrictStr], Field(description="Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list")] = None,
        var_from: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        until: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        last_updated_from: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        last_updated_until: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        size: Annotated[Optional[StrictInt], Field(description="Range is: [1, 1000]")] = None,
        cursor: Annotated[Optional[StrictStr], Field(description="Continue scrolling from cursor")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetCredResponseStream]:
        """Credential stream

        Returns list of Credentials matching filter criteria. The response contains “cursor_next” field, which should be provided to the next subsequent call to fetch potential next page of the results. Results are sorted by ascending order of the last_updated_ts field. Two indications of the end of the stream are possible: the result list is empty or its size is less than the requested items count.

        :param credential_set_name: Search by credential set name.
        :type credential_set_name: str
        :param credential_set_id: Search by credential set id.
        :type credential_set_id: str
        :param domain: Search by credential domain(detection domain).
        :type domain: str
        :param affiliation_group: Possible values: my_employees, my_customers, third_parties, vip_emails
        :type affiliation_group: str
        :param password_strength: Possible values: excellent, strong, medium, weak, poor, not_provided
        :type password_strength: str
        :param password_length_gte: Must be greater than or equal to 0
        :type password_length_gte: int
        :param password_lowercase_gte: Must be greater than or equal to 0
        :type password_lowercase_gte: int
        :param password_uppercase_gte: Must be greater than or equal to 0
        :type password_uppercase_gte: int
        :param password_numbers_gte: Must be greater than or equal to 0
        :type password_numbers_gte: int
        :param password_punctuation_gte: Must be greater than or equal to 0
        :type password_punctuation_gte: int
        :param password_symbols_gte: Must be greater than or equal to 0
        :type password_symbols_gte: int
        :param password_separators_gte: Must be greater than or equal to 0
        :type password_separators_gte: int
        :param password_other_gte: Must be greater than or equal to 0
        :type password_other_gte: int
        :param password_entropy_gte: Must be greater than or equal to 0
        :type password_entropy_gte: int
        :param password_plain: Must be URL encoded
        :type password_plain: str
        :param credential_login: Search by credential login.
        :type credential_login: str
        :param detected_malware: Search by credential detected malware. For example: agent_tesla, Lumma, VIDAR
        :type detected_malware: str
        :param girs: Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list
        :type girs: str
        :param var_from: UNIX timestamp(in milliseconds)
        :type var_from: int
        :param until: UNIX timestamp(in milliseconds)
        :type until: int
        :param last_updated_from: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_from: str
        :param last_updated_until: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_until: str
        :param size: Range is: [1, 1000]
        :type size: int
        :param cursor: Continue scrolling from cursor
        :type cursor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credentials_stream_serialize(
            credential_set_name=credential_set_name,
            credential_set_id=credential_set_id,
            domain=domain,
            affiliation_group=affiliation_group,
            password_strength=password_strength,
            password_length_gte=password_length_gte,
            password_lowercase_gte=password_lowercase_gte,
            password_uppercase_gte=password_uppercase_gte,
            password_numbers_gte=password_numbers_gte,
            password_punctuation_gte=password_punctuation_gte,
            password_symbols_gte=password_symbols_gte,
            password_separators_gte=password_separators_gte,
            password_other_gte=password_other_gte,
            password_entropy_gte=password_entropy_gte,
            password_plain=password_plain,
            credential_login=credential_login,
            detected_malware=detected_malware,
            girs=girs,
            var_from=var_from,
            until=until,
            last_updated_from=last_updated_from,
            last_updated_until=last_updated_until,
            size=size,
            cursor=cursor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredResponseStream",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_credentials_stream_without_preload_content(
        self,
        credential_set_name: Annotated[Optional[StrictStr], Field(description="Search by credential set name.")] = None,
        credential_set_id: Annotated[Optional[StrictStr], Field(description="Search by credential set id.")] = None,
        domain: Annotated[Optional[StrictStr], Field(description="Search by credential domain(detection domain).")] = None,
        affiliation_group: Annotated[Optional[StrictStr], Field(description="Possible values: my_employees, my_customers, third_parties, vip_emails")] = None,
        password_strength: Annotated[Optional[StrictStr], Field(description="Possible values: excellent, strong, medium, weak, poor, not_provided")] = None,
        password_length_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_lowercase_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_uppercase_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_numbers_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_punctuation_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_symbols_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_separators_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_other_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_entropy_gte: Annotated[Optional[StrictInt], Field(description="Must be greater than or equal to 0")] = None,
        password_plain: Annotated[Optional[StrictStr], Field(description="Must be URL encoded")] = None,
        credential_login: Annotated[Optional[StrictStr], Field(description="Search by credential login.")] = None,
        detected_malware: Annotated[Optional[StrictStr], Field(description="Search by credential detected malware. For example: agent_tesla, Lumma, VIDAR")] = None,
        girs: Annotated[Optional[StrictStr], Field(description="Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list")] = None,
        var_from: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        until: Annotated[Optional[StrictInt], Field(description="UNIX timestamp(in milliseconds)")] = None,
        last_updated_from: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        last_updated_until: Annotated[Optional[StrictStr], Field(description="UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days")] = None,
        size: Annotated[Optional[StrictInt], Field(description="Range is: [1, 1000]")] = None,
        cursor: Annotated[Optional[StrictStr], Field(description="Continue scrolling from cursor")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Credential stream

        Returns list of Credentials matching filter criteria. The response contains “cursor_next” field, which should be provided to the next subsequent call to fetch potential next page of the results. Results are sorted by ascending order of the last_updated_ts field. Two indications of the end of the stream are possible: the result list is empty or its size is less than the requested items count.

        :param credential_set_name: Search by credential set name.
        :type credential_set_name: str
        :param credential_set_id: Search by credential set id.
        :type credential_set_id: str
        :param domain: Search by credential domain(detection domain).
        :type domain: str
        :param affiliation_group: Possible values: my_employees, my_customers, third_parties, vip_emails
        :type affiliation_group: str
        :param password_strength: Possible values: excellent, strong, medium, weak, poor, not_provided
        :type password_strength: str
        :param password_length_gte: Must be greater than or equal to 0
        :type password_length_gte: int
        :param password_lowercase_gte: Must be greater than or equal to 0
        :type password_lowercase_gte: int
        :param password_uppercase_gte: Must be greater than or equal to 0
        :type password_uppercase_gte: int
        :param password_numbers_gte: Must be greater than or equal to 0
        :type password_numbers_gte: int
        :param password_punctuation_gte: Must be greater than or equal to 0
        :type password_punctuation_gte: int
        :param password_symbols_gte: Must be greater than or equal to 0
        :type password_symbols_gte: int
        :param password_separators_gte: Must be greater than or equal to 0
        :type password_separators_gte: int
        :param password_other_gte: Must be greater than or equal to 0
        :type password_other_gte: int
        :param password_entropy_gte: Must be greater than or equal to 0
        :type password_entropy_gte: int
        :param password_plain: Must be URL encoded
        :type password_plain: str
        :param credential_login: Search by credential login.
        :type credential_login: str
        :param detected_malware: Search by credential detected malware. For example: agent_tesla, Lumma, VIDAR
        :type detected_malware: str
        :param girs: Filter results by custom GIRs (General Intelligence Requirements), my_girs or company_pirs. Using multiple values will return results based on the aggregated GIR list
        :type girs: str
        :param var_from: UNIX timestamp(in milliseconds)
        :type var_from: int
        :param until: UNIX timestamp(in milliseconds)
        :type until: int
        :param last_updated_from: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_from: str
        :param last_updated_until: UNIX timestamp(in milliseconds) or period e.g. 24hours or 7days
        :type last_updated_until: str
        :param size: Range is: [1, 1000]
        :type size: int
        :param cursor: Continue scrolling from cursor
        :type cursor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_credentials_stream_serialize(
            credential_set_name=credential_set_name,
            credential_set_id=credential_set_id,
            domain=domain,
            affiliation_group=affiliation_group,
            password_strength=password_strength,
            password_length_gte=password_length_gte,
            password_lowercase_gte=password_lowercase_gte,
            password_uppercase_gte=password_uppercase_gte,
            password_numbers_gte=password_numbers_gte,
            password_punctuation_gte=password_punctuation_gte,
            password_symbols_gte=password_symbols_gte,
            password_separators_gte=password_separators_gte,
            password_other_gte=password_other_gte,
            password_entropy_gte=password_entropy_gte,
            password_plain=password_plain,
            credential_login=credential_login,
            detected_malware=detected_malware,
            girs=girs,
            var_from=var_from,
            until=until,
            last_updated_from=last_updated_from,
            last_updated_until=last_updated_until,
            size=size,
            cursor=cursor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCredResponseStream",
            '400': "BadRequest",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "NotFound",
            '409': "Conflict",
            '500': "InternalServerError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_credentials_stream_serialize(
        self,
        credential_set_name,
        credential_set_id,
        domain,
        affiliation_group,
        password_strength,
        password_length_gte,
        password_lowercase_gte,
        password_uppercase_gte,
        password_numbers_gte,
        password_punctuation_gte,
        password_symbols_gte,
        password_separators_gte,
        password_other_gte,
        password_entropy_gte,
        password_plain,
        credential_login,
        detected_malware,
        girs,
        var_from,
        until,
        last_updated_from,
        last_updated_until,
        size,
        cursor,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if credential_set_name is not None:
            
            _query_params.append(('credential_set_name', credential_set_name))
            
        if credential_set_id is not None:
            
            _query_params.append(('credential_set_id', credential_set_id))
            
        if domain is not None:
            
            _query_params.append(('domain', domain))
            
        if affiliation_group is not None:
            
            _query_params.append(('affiliation_group', affiliation_group))
            
        if password_strength is not None:
            
            _query_params.append(('password_strength', password_strength))
            
        if password_length_gte is not None:
            
            _query_params.append(('password_length_gte', password_length_gte))
            
        if password_lowercase_gte is not None:
            
            _query_params.append(('password_lowercase_gte', password_lowercase_gte))
            
        if password_uppercase_gte is not None:
            
            _query_params.append(('password_uppercase_gte', password_uppercase_gte))
            
        if password_numbers_gte is not None:
            
            _query_params.append(('password_numbers_gte', password_numbers_gte))
            
        if password_punctuation_gte is not None:
            
            _query_params.append(('password_punctuation_gte', password_punctuation_gte))
            
        if password_symbols_gte is not None:
            
            _query_params.append(('password_symbols_gte', password_symbols_gte))
            
        if password_separators_gte is not None:
            
            _query_params.append(('password_separators_gte', password_separators_gte))
            
        if password_other_gte is not None:
            
            _query_params.append(('password_other_gte', password_other_gte))
            
        if password_entropy_gte is not None:
            
            _query_params.append(('password_entropy_gte', password_entropy_gte))
            
        if password_plain is not None:
            
            _query_params.append(('password_plain', password_plain))
            
        if credential_login is not None:
            
            _query_params.append(('credential_login', credential_login))
            
        if detected_malware is not None:
            
            _query_params.append(('detected_malware', detected_malware))
            
        if girs is not None:
            
            _query_params.append(('girs', girs))
            
        if var_from is not None:
            
            _query_params.append(('from', var_from))
            
        if until is not None:
            
            _query_params.append(('until', until))
            
        if last_updated_from is not None:
            
            _query_params.append(('last_updated_from', last_updated_from))
            
        if last_updated_until is not None:
            
            _query_params.append(('last_updated_until', last_updated_until))
            
        if size is not None:
            
            _query_params.append(('size', size))
            
        if cursor is not None:
            
            _query_params.append(('cursor', cursor))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/integrations/creds/v1/credentials/stream',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





import abc
from dataclasses import dataclass
import datetime
import logging
import re
from abc import ABC
from collections.abc import Callable
from typing import Union, List, Optional, Any
from enum import Enum

from stix2 import Bundle

from .. import INTEL_471, STIXMapperSettings
from ..exceptions import EmptyBundle, StixMapperNotFound

log = logging.getLogger(__name__)


@dataclass
class MappingConfig:
    entities_mapper: Callable
    kwargs_extractor: Callable
    name_extractor: Callable
    opencti_type: str


@dataclass
class Link:
    type: str
    href: str
    description: Optional[str] = None


class StixMapper:
    def __init__(self, settings: STIXMapperSettings = None):
        self.settings = settings if settings else STIXMapperSettings()

    mappers = {}

    @classmethod
    def register(cls, name: str, condition) -> Callable:
        """
        Decorator used for registering mapper classes. Decorate any class derived from BaseMapper like this:

        @StixMapper.register("actors", lambda x: "actorTotalCount" in x)
        class ActorsMapper(BaseMapper):
            def map(self, source: dict) -> Bundle:
                ... my implementation ...

        @param name: unique name under which the mapper will be registered
        @param condition: callable that will be called against the source dict
                          to determine if given mapper should be used or not

        """

        def inner_wrapper(wrapped_class: Callable) -> Callable:
            if name in cls.mappers:
                log.info("Mapper for %s already exists. Will replace it", name)
            cls.mappers[name] = (condition, wrapped_class)
            return wrapped_class

        return inner_wrapper

    def map(self, source: dict, stix_version: str = "2.1") -> Bundle:
        log.info("Initializing converter. STIX version %s.", stix_version)
        for name, (condition, mapper_class) in self.mappers.items():
            if condition(source):
                log.info("Mapping Verity471 payload for %s.", name)
                mapper = mapper_class(self.settings)
                bundle = mapper.map(self._deenum(source))
                if bundle:
                    return bundle
                else:
                    raise EmptyBundle("STIX Mapper produced an empty bundle.")
        raise StixMapperNotFound(
            f"STIX Mapper for this payload is not available (keys: {', '.join(source.keys())})."
        )

    @classmethod
    def _deenum(cls, obj: Any) -> Any:
        # Enum -> unwrap to the underlying value
        if isinstance(obj, Enum):
            return obj.value

        # dict -> recurse on values
        if isinstance(obj, dict):
            return {k: cls._deenum(v) for k, v in obj.items()}

        # list/tuple/set -> recurse
        if isinstance(obj, list):
            return [cls._deenum(v) for v in obj]
        if isinstance(obj, tuple):
            return tuple(cls._deenum(v) for v in obj)
        if isinstance(obj, set):
            return {cls._deenum(v) for v in obj}

        # anything else -> keep
        return obj


class BaseMapper(ABC):
    def __init__(self, settings: STIXMapperSettings):
        self.now = datetime.datetime.now(datetime.timezone.utc)
        self.settings = settings

    @abc.abstractmethod
    def map(self, source: dict) -> Bundle:
        raise NotImplementedError

    def map_confidence(self, confidence: Union[str, None]):
        # There are two confidence scales used in Verity471: https://en.wikipedia.org/wiki/Admiralty_code and low/medium/high
        # This function handles both according to
        # [STIX confidence scales](https://docs.oasis-open.org/cti/stix/v2.1/os/stix-v2.1-os.html#_1v6elyto0uqg)

        # If it's Admiralty_code we're interested in the second part only (Credibility), which is a number between 1 and 6.
        value = re.sub(r"^[A-F]([1-6])$", "\\1", confidence or "", flags=re.IGNORECASE)

        # If there's no match, we expect a word from low/medium/high scale.
        # If for any reason it's not the case either, we set confidence as not specified (`None`)
        return {
            "6": 0, "5": 10, "4": 30, "3": 50, "2": 70, "1": 90,
            "low": 15, "medium": 50, "high": 85
        }.get(value, 0)

    def map_links(self, links_sources: Union[dict, list[dict]]) -> list[Link]:
        links = []
        if isinstance(links_sources, dict):
            links_sources = [links_sources]
        for links_source in links_sources:
            for key, value in links_source.items():
                links.append(Link(type=key, href=value.get('href')))
        return links

    def get_girs_labels(self, gir_paths: List[dict]):
        return [f'{INTEL_471} - GIR {i["path"]} - {i["name"]}' for i in gir_paths]



import re

from pycti import CustomObservableCryptocurrencyWallet
from stix2 import URL, IPv4Address, DomainName, File, AutonomousSystem, UserAccount, IPv6Address, EmailAddress
from stix2.exceptions import InvalidValueError

from verity471.verity_stix import author_identity
from verity471.verity_stix.constants import MARKING, X_OPENCTI_CREATED_BY, X_OPENCTI_LABELS, PLATFORM_VERITY471


def map_url(value: str, *args, **kwargs) -> URL:
    return URL(
        value=value,
        object_marking_refs=[MARKING],
        custom_properties={
            X_OPENCTI_CREATED_BY: author_identity.id,
            X_OPENCTI_LABELS: [PLATFORM_VERITY471]
            }
    )


def map_ipv4(value: str, *args, **kwargs) -> IPv4Address:
    return IPv4Address(
        value=value,
        object_marking_refs=[MARKING],
        custom_properties={
            X_OPENCTI_CREATED_BY: author_identity.id,
            X_OPENCTI_LABELS: [PLATFORM_VERITY471]
            }
    )


def map_ipv6(value: str, *args, **kwargs) -> IPv6Address:
    return IPv6Address(
        value=value,
        object_marking_refs=[MARKING],
        custom_properties={
            X_OPENCTI_CREATED_BY: author_identity.id,
            X_OPENCTI_LABELS: [PLATFORM_VERITY471]
            }
    )


def map_domain(value: str, *args, **kwargs) -> DomainName:
    if value.startswith(("http://", "https://")):
        raise InvalidValueError(DomainName, "value", f"'{value}' is not valid domain name")
    return DomainName(
        value=value,
        object_marking_refs=[MARKING],
        custom_properties={
            X_OPENCTI_CREATED_BY: author_identity.id,
            X_OPENCTI_LABELS: [PLATFORM_VERITY471]
            }
    )


def map_email_address(value: str, *args, **kwargs) -> EmailAddress:
    return EmailAddress(
        value=value,
        object_marking_refs=[MARKING],
        custom_properties={
            X_OPENCTI_CREATED_BY: author_identity.id,
            X_OPENCTI_LABELS: [PLATFORM_VERITY471]
            }
    )


def map_autonomous_system(value: str, *args, **kwargs) -> AutonomousSystem:
    return AutonomousSystem(
        number=int(re.sub(r"[A-Z]", "", value)),
        object_marking_refs=[MARKING],
        custom_properties={
            X_OPENCTI_CREATED_BY: author_identity.id,
            X_OPENCTI_LABELS: [PLATFORM_VERITY471]
            }
    )


def map_crypto_wallet(value: str, *args, **kwargs) -> CustomObservableCryptocurrencyWallet:
    return CustomObservableCryptocurrencyWallet(
        value=value,
        object_marking_refs=[MARKING],
        custom_properties={
            X_OPENCTI_CREATED_BY: author_identity.id,
            X_OPENCTI_LABELS: [PLATFORM_VERITY471]
            }
    )


def map_user_account(value: str, type: str, *args, **kwargs) -> UserAccount:
    return UserAccount(
        account_type=type,
        user_id=value,
        object_marking_refs=[MARKING],
        custom_properties={
            X_OPENCTI_CREATED_BY: author_identity.id,
            X_OPENCTI_LABELS: [PLATFORM_VERITY471]
            }
    )


def map_file(md5: str = None, sha1: str = None, sha256: str = None, name: str = None) -> File:
    hashes = {}
    if md5:
        hashes["MD5"] = md5
    if sha1:
        hashes["SHA-1"] = sha1
    if sha256:
        hashes["SHA-256"] = sha256

    file_kwargs = {
        "object_marking_refs": [MARKING],
        "custom_properties": {
            X_OPENCTI_CREATED_BY: author_identity.id,
            X_OPENCTI_LABELS: [PLATFORM_VERITY471]
            }
    }
    if hashes:
        file_kwargs["hashes"] = hashes
    if name:
        file_kwargs["name"] = name

    return File(**file_kwargs)


def map_file_hash(value: str, type: str, *args, **kwargs) -> File:
    kwargs = {type.lower(): value}
    return map_file(**kwargs)


def map_filename(value: str, *args, **kwargs) -> File:
    return map_file(name=value)
